#include "gamepadmonitor.h"
#include <QDebug>

GamepadMonitor::GamepadMonitor(QObject *parent) : QObject(parent)
{
    m_gamepadManager = QGamepadManager::instance();

    rollScale = 20;  //-20 to 20 deg/s
    pitchScale = 20;
    yawScale = 30;
    thrustScale = 30000; //0 to 60,000 thrust

    connect(m_gamepadManager, SIGNAL(gamepadConnected(int)), this, SLOT(gamepadConnectedHandler(int)));
    connect(m_gamepadManager, SIGNAL(gamepadDisconnected(int)), this, SLOT(gamepadDisconnectedHandler(int)));

    auto gamepads = m_gamepadManager->connectedGamepads();
    if (gamepads.isEmpty()) {
        // Did not find any connected gamepads creating default gamepad;
        m_gamepad = new QGamepad();
        return;
    }

    m_gamepad = new QGamepad(*gamepads.begin(), this);
    qInfo() << "gamepad is connected, deviceId = " << m_gamepad->deviceId();
}

void GamepadMonitor::gamepadConnectedHandler(int deviceId){
    emit(gamepadConnected());
    qInfo() << "gamepad is connected, deviceId = " << deviceId;
    m_gamepad = new QGamepad(deviceId, this);
}

void GamepadMonitor::gamepadDisconnectedHandler(int deviceId){
    emit(gamepadDisconnected());
    qInfo() << "gamepad disconnected";
}

bool GamepadMonitor::isGamepadConnected(){
    return m_gamepad->isConnected();
}

void GamepadMonitor::resetConfig(){
    qInfo() << "gamepad config reset";
    m_gamepadManager->resetConfiguration(m_gamepad->deviceId());
}

void GamepadMonitor::configureAxis(QString axisName){
    if(axisName == "roll"){
        m_gamepadManager->configureAxis(m_gamepad->deviceId(), QGamepadManager::AxisRightX);
        qInfo() << "roll configured";
    }else if(axisName == "yaw"){
        m_gamepadManager->configureAxis(m_gamepad->deviceId(), QGamepadManager::AxisLeftX);
        qInfo() << "yaw configured";
    }else if(axisName == "pitch"){
        m_gamepadManager->configureAxis(m_gamepad->deviceId(), QGamepadManager::AxisRightY);
        qInfo() << "pitch configured";
    }else if(axisName == "thrust"){
        m_gamepadManager->configureAxis(m_gamepad->deviceId(), QGamepadManager::AxisLeftY);
        qInfo() << "thrust configured";
    }
}

float GamepadMonitor::getRoll(){
    return (float) m_gamepad->axisRightX() * rollScale;
}

float GamepadMonitor::getYaw(){
    return (float) m_gamepad->axisLeftX() * yawScale;
}

float GamepadMonitor::getPitch(){
    return (float) m_gamepad->axisRightY() * pitchScale;
}

float GamepadMonitor::getThrust(){
    return (float) (m_gamepad->axisLeftY() + 1) * thrustScale;
}

GamepadMonitor::~GamepadMonitor(){

}
