#ifndef MAINWINDOW_H
#define MAINWINDOW_H

#include <QMainWindow>
#include <QStringList>
#include <QStandardItemModel>
#include <QGraphicsScene>
#include "quaditem.h"
#include "gamepadmonitor.h"
#include "crazyflieworker.h"


namespace Ui {
class MainWindow;
}

class MainWindow : public QMainWindow
{
    Q_OBJECT

public:
    explicit MainWindow(QWidget *parent = 0);
    ~MainWindow();

signals:
    void connectWorkers();
    void disconnectWorkers();
    void getParamValue(QString node, QString param);
    void getParamValue(QString key);
    void getNodeOutput(QString node);
    void setParamValue(QString node, QString param, float value);
    void setParamValue(QString key, double value);
    void getPosAttFromBackend();
    void rateSetpointSignal(float roll, float pitch, float yaw, float throttle);
    void angleSetpointSignal(float roll, float pitch, float yaw, float throttle);
    void getGroupEntries(int box, QString gName);

private slots:
    void on_pbStart_clicked();

    void on_pbConnect_clicked();

    void on_pbStop_clicked();

    void on_chooseBackend_clicked();

    void updateTracker(float x, float y, float z, float p, float r, float yaw);

    void updatePosAtt();

    void newNodes(QStringList blocks);
    void newParams(QStringList params);
    void newParamValue(QString node, QString param, float val);
    void newNodeOutput(QString node, float output);
    void newConstantBlocks(QStringList blocks);
    void newGroupEntries(int box, QStringList entries);

    void on_pb_getParam_click();

    void on_pb_setParam_click();

    void on_paramSelect_currentIndexChanged(const QString &arg1);

    void on_paramValue_returnPressed();

    void on_pbAppendSetpoint_clicked();

    void on_pbNextSetpoint_clicked();

    void sendSetpoints();
    void sendSelectedSetpoint();

    void on_pbActualToSetpoint_clicked();

    void on_pbDeleteSetpoint_clicked();

    void on_pbActualToWaypoint_clicked();

    void on_pbMoveUp_clicked();

    void on_pbMoveDown_clicked();

    void on_pbInsertSetpoint_clicked();

    void on_pbSaveWaypoints_clicked();

    void on_pbLoadWaypoints_clicked();

    void on_xSetpoint_returnPressed();

    void on_run_data_analysis_button_clicked();

    void on_ySetpoint_returnPressed();

    void on_zSetpoint_returnPressed();

    void on_yawSetpoint_returnPressed();

    void backendStarted();
    void connectMatlab();
    void backendFinished(int exitCode, QProcess::ExitStatus exitStatus);
    void backendError(QProcess::ProcessError error);
    void backendRead();
    void workerConnected();
    void workerDisconnected();

    void on_applySetpointButton_clicked();

    void on_stopSetpointButton_clicked();
    void setParamGroups(QStringList groupNames);
    void getGroupBoxChange(int index);
    void getGroupBox2Change(int index);
    void displayParamValue(QString param, float value);
    void getEntryBoxChange(int index);
    void getEntryBoxChange2(int index);

    void onGamepadDisconnect();
    void onGamepadConnect();
    void on_pb_configRoll_clicked();
    void updateGamepad();

    void on_pb_resetConfig_clicked();

    void on_pb_configYaw_clicked();

    void on_pb_configThrust_clicked();

    void on_pb_configPitch_clicked();

    void on_tActual_sliderMoved(int position);

    void on_rbGamepadControl_toggled(bool checked);

    void on_rbManualSetpoint_toggled(bool checked);

private:
    Ui::MainWindow *ui;
    QStandardItemModel * setpointList;
    QTimer * nextSpTimer;
    float sp_roll;
    float sp_pitch;
    float sp_yaw;
    float sp_thrust;
    QTimer * trackerTimer;
    QTimer * workerStartTimer;
    QTimer * crazyflieTimer;
    QTimer *pollGamepadTimer;
    QProcess * backendProcess;
    QProcess * matlabProcess;
    int connectedWorkers;
    GamepadMonitor *gamepadMonitor;
    CrazyflieWorker *crazyflieWorker;
};

#endif // MAINWINDOW_H
